@@ -1,9 +1,8 @@
-from inspect import isawaitable
-from typing import Any, Callable
+from collections.abc import Mapping
+from inspect import iscoroutinefunction
+from typing import Any, Callable, Dict, Optional
 
-from graphql.error import GraphQLError
-from graphql.pyutils import AwaitableOrValue, inspect
-from graphql.type import (
+from graphql import (
     GraphQLArgument,
     GraphQLField,
     GraphQLFieldMap,
@@ -16,7 +15,7 @@ from graphql.type import (
     GraphQLString,
     Thunk,
 )
-
+from graphql.pyutils import AwaitableOrValue
 
 # Note: Contrary to the Javascript implementation of MutationFn,
 # the context is passed as part of the GraphQLResolveInfo and any arguments
@@ -31,13 +30,19 @@ def resolve_maybe_thunk(thing_or_thunk: Thunk) -> Any:
     return thing_or_thunk() if callable(thing_or_thunk) else thing_or_thunk
 
 
+class NullResult:
+    def __init__(self, clientMutationId: Optional[str] = None) -> None:
+        self.clientMutationId = clientMutationId
+
+
 def mutation_with_client_mutation_id(
     name: str,
     input_fields: Thunk[GraphQLInputFieldMap],
     output_fields: Thunk[GraphQLFieldMap],
     mutate_and_get_payload: MutationFn,
-    description: str = None,
-    deprecation_reason: str = None,
+    description: Optional[str] = None,
+    deprecation_reason: Optional[str] = None,
+    extensions: Optional[Dict[str, Any]] = None,
 ) -> GraphQLField:
     """
     Returns a GraphQLFieldConfig for the specified mutation.
@@ -57,32 +62,48 @@ def mutation_with_client_mutation_id(
     def augmented_input_fields() -> GraphQLInputFieldMap:
         return dict(
             resolve_maybe_thunk(input_fields),
-            clientMutationId=GraphQLInputField(GraphQLNonNull(GraphQLString)),
+            clientMutationId=GraphQLInputField(GraphQLString),
         )
 
     def augmented_output_fields() -> GraphQLFieldMap:
         return dict(
             resolve_maybe_thunk(output_fields),
-            clientMutationId=GraphQLField(GraphQLNonNull(GraphQLString)),
+            clientMutationId=GraphQLField(GraphQLString),
         )
 
     output_type = GraphQLObjectType(name + "Payload", fields=augmented_output_fields)
 
     input_type = GraphQLInputObjectType(name + "Input", fields=augmented_input_fields)
 
-    # noinspection PyShadowingBuiltins
-    async def resolve(_root, info, input):
-        payload = mutate_and_get_payload(info, **input)
-        if isawaitable(payload):
-            payload = await payload
-        try:
-            payload.clientMutationId = input["clientMutationId"]
-        except KeyError:
-            raise GraphQLError(
-                "Cannot set clientMutationId"
-                f" in the payload object {inspect(payload)}."
-            )
-        return payload
+    if iscoroutinefunction(mutate_and_get_payload):
+
+        # noinspection PyShadowingBuiltins
+        async def resolve(_root: Any, info: GraphQLResolveInfo, input: Dict) -> Any:
+            payload = await mutate_and_get_payload(info, **input)
+            clientMutationId = input.get("clientMutationId")
+            if payload is None:
+                return NullResult(clientMutationId)
+            if isinstance(payload, Mapping):
+                payload["clientMutationId"] = clientMutationId  # type: ignore
+            else:
+                payload.clientMutationId = clientMutationId
+            return payload
+
+    else:
+
+        # noinspection PyShadowingBuiltins
+        def resolve(  # type: ignore
+            _root: Any, info: GraphQLResolveInfo, input: Dict
+        ) -> Any:
+            payload = mutate_and_get_payload(info, **input)
+            clientMutationId = input.get("clientMutationId")
+            if payload is None:
+                return NullResult(clientMutationId)
+            if isinstance(payload, Mapping):
+                payload["clientMutationId"] = clientMutationId  # type: ignore
+            else:
+                payload.clientMutationId = clientMutationId  # type: ignore
+            return payload
 
     return GraphQLField(
         output_type,
@@ -90,4 +111,5 @@ def mutation_with_client_mutation_id(
         deprecation_reason=deprecation_reason,
         args={"input": GraphQLArgument(GraphQLNonNull(input_type))},
         resolve=resolve,
+        extensions=extensions,
     )
